@@ -1,10 +1,10 @@
-#define RUBY_VERSION "2.5.9"
-#define RUBY_RELEASE_DATE "2021-04-05"
-#define RUBY_PATCHLEVEL 229
+#define RUBY_VERSION "2.6.1"
+#define RUBY_RELEASE_DATE RUBY_RELEASE_YEAR_STR"-"RUBY_RELEASE_MONTH_STR"-"RUBY_RELEASE_DAY_STR
+#define RUBY_PATCHLEVEL 33
 
-#define RUBY_RELEASE_YEAR 2021
-#define RUBY_RELEASE_MONTH 4
-#define RUBY_RELEASE_DAY 5
+#define RUBY_RELEASE_YEAR 2019
+#define RUBY_RELEASE_MONTH 1
+#define RUBY_RELEASE_DAY 30
 
 #include "ruby/version.h"
 
@@ -60,11 +60,11 @@
 # define RUBY_REVISION_STR ""
 #endif
 
-# define RUBY_DESCRIPTION	    \
+# define RUBY_DESCRIPTION_WITH(opt) \
     "ruby "RUBY_VERSION		    \
     RUBY_PATCHLEVEL_STR		    \
     " ("RUBY_RELEASE_DATE	    \
-    RUBY_REVISION_STR") "	    \
+    RUBY_REVISION_STR")"opt" "	    \
     "["RUBY_PLATFORM"]"
 # define RUBY_COPYRIGHT		    \
     "ruby - Copyright (C) "	    \
